
# m4 macros from the Unidata netcdf 2.3.2 pl4 distribution. Modified for use
# with GNU Autoconf 2.1. I renamed these from UC_* to DODS_* so that there
# will not be confusion when porting future versions of netcdf into the DODS
# source distribution. Unidata, Inc. wrote the text of these macros and holds
# a copyright on them.
#
# jhrg 3/27/95
#
# Added some of my own macros (don't blame Unidata for them!) starting with
# DODS_PROG_LEX and down in the file. jhrg 2/11/96
#
# I've added a table of contents for this file. jhrg 2/2/98
#
# Modified for use with autoconf 2.57 and the Ocapi. This copy of the macros
# in DODS/etc/aclocal.m4 (named acinclude.m4) is used to _generate_ the Ocapi's
# local copy of aclocal.m4 using the autoconf tool 'aclocal'. Confusing, I
# know. I removed macros I thought would be of little or no use. jhrg 1/25/05
# 
# 1. Unidata-derived macros. 
# 2. Macros for finding libraries used by the core software.
# 3. Macros used to test things about the compiler
# 4. Macros for locating various systems (Matlab, etc.)
# 5. Macros used to test things about the computer/OS/hardware
#
# $Id: acinclude.m4 12617 2005-11-10 10:38:44Z jimg $

# 1. Unidata-derived macros

# 2. Libraries

# 3. Compiler test macros
#--------------------------------------------------------------------------

# These macros used to test for various versions; we don't use them anymore.
# 09/20/02 jhrg

AC_DEFUN([DODS_PROG_LEX], [dnl
    AC_PROG_LEX])

AC_DEFUN([DODS_PROG_BISON], [dnl
    AC_CHECK_PROG(YACC,bison,bison)])

AC_DEFUN([DODS_GCC_VERSION], [dnl
    AC_MSG_CHECKING(for gcc/g++ 2.8 or greater)

    GCC_VER=`gcc -v 2>&1 | awk '/version/ {print}'`

    dnl We need the gcc version number as a number, without `.'s and limited
    dnl to three digits. The old version below was replaced by Andy Jacobson's 
    dnl patch which works with gcc 3, including the pre-release versions.

    dnl GCC_VER=`echo $GCC_VER | sed 's@[[a-z ]]*\([[0-9.]]\)@\1@'`
    GCC_VER=`echo $GCC_VER | sed 's@.*gcc version \([[0-9\.]]*\).*@\1@'`

    case $GCC_VER in
        *egcs*) AC_MSG_RESULT(Found egcs version ${GCC_VER}.) ;;
	2.[[7-9]]*)   AC_MSG_RESULT(Found gcc/g++ version ${GCC_VER}) ;;
	3.[[0-9]]*)   AC_MSG_RESULT(Found gcc/g++ version ${GCC_VER}) ;;
        *)      AC_MSG_ERROR(must be at least version 2.7.x) ;;
    esac])

# 4. Macros to locate various programs/systems used by parts of DODS
#---------------------------------------------------------------------------

# Added by Ethan, 1999/06/21
# Look for GNU tar.
# 
# I modified the regexp below but it still does not work exactly correctly; 
# the variable tar_ver should have only the version number in it. However,
# my version (1.12) spits out a multi-line thing. The regexp below gets the
# version number from the first line but does not remove the subsequent lines
# of garbage. 7/15/99 jhrg
# Added awk line to handle multiline output. 1999/07/22 erd

AC_DEFUN([DODS_PROG_GTAR], [dnl
    AC_CHECK_PROGS(TAR,gtar tar,tar)
    case "$TAR" in
	*tar)
	    tar_ver=`$TAR --version 2>&1 | awk '/G[[Nn]][[Uu]] tar/ {print}'`
	    tar_ver=`echo $tar_ver | sed 's/.*GNU tar[[^0-9.]]*\([[0-9._]]*\)/\1/'`
	    if test -n "$tar_ver"
	    then
		AC_MSG_RESULT(Found Gnu tar version ${tar_ver}.)
	    else
		AC_MSG_WARN(GNU tar is required for some Makefile targets.)
	    fi
	    ;;
	*)
	    AC_MSG_WARN(GNU tar is required for some Makefile targets.)
	    ;;
    esac

    AC_SUBST(TAR)])

# Find the matlab root directory
# cross-compile problem with test option -d

AC_DEFUN([DODS_MATLAB], [dnl
    AC_ARG_WITH(matlab,
        [  --with-matlab=ARG       Where is the Matlab root directory],
        MATLAB_ROOT=${withval}, MATLAB_ROOT="$MATLAB_ROOT")
    if test "$MATLAB_ROOT" = no; then
        MATLAB_ROOT="$MATLAB_ROOT"
    elif test ! -d "$MATLAB_ROOT"
    then
        MATLAB_ROOT=""
    fi
    if test -z "$MATLAB_ROOT"
    then
        AC_MSG_CHECKING(for matlab root)

	MATLAB_ROOT=`mex -v 2>&1 | awk '/MATLAB *= / {print}'`
	MATLAB_ROOT=`echo $MATLAB_ROOT | sed 's@[[^/]]*\(/.*\)@\1@'`

	if test -z "$MATLAB_ROOT"
	then
	    AC_MSG_ERROR(Matlab not found! Run configure --help)
        else
	    AC_SUBST(MATLAB_ROOT)
	    AC_MSG_RESULT($MATLAB_ROOT)
        fi
    else
        AC_SUBST(MATLAB_ROOT)
        AC_MSG_RESULT("Set Matlab root to $MATLAB_ROOT")
    fi

    dnl Find the lib directory (which is named according to machine type).
    dnl The test was using -z; I changed it to -n to fix a problem withthe 
    dnl matlab server build. The build did not find the libraries with the
    dnl env var MATLAB_LIB was not set. 04/12/04 jhrg
    AC_MSG_CHECKING(for matlab library dir)
    if test -n "$MATLAB_LIB"
    then
        matlab_lib_dir=${MATLAB_LIB}
    else
        matlab_lib_dir=`find ${MATLAB_ROOT}/extern/lib -name 'libmat*' -print | sed '1q'`
        matlab_lib_dir=`echo $matlab_lib_dir | sed 's@\(.*\)/libmat.*@\1@'`
    fi
    AC_MSG_RESULT($matlab_lib_dir)

    if test "$matlab_lib_dir"
    then
	LDFLAGS="$LDFLAGS -L$matlab_lib_dir"
	dnl This is used by the nph script to set LD_LIBRARY_PATH
	AC_SUBST(matlab_lib_dir)
    fi
    
    dnl sleazy test for version 5; look for the version 4 compat flag

    if grep V4_COMPAT ${MATLAB_ROOT}/extern/include/mat.h > /dev/null 2>&1
    then
	dnl if we're here, we're linking under ML 5 or 6. 02/10/03 jhrg
	MAT_VERSION_FLAG="-V4"
	dnl ML 6 lacks libmi.a/.so. 02/10/03 jhrg
	if access -r ${matlab_lib_dir}/libmi.*
	then
	    MATLIBS="-lmat -lmi -lmx -lut"
	else
	    MATLIBS="-lmat -lmx -lut"
	fi
    else
       MAT_VERSION_FLAG=""
       MATLIBS="-lmat"
    fi

    AC_CHECK_LIB(ots, _OtsDivide64Unsigned, MATLIBS="$MATLIBS -lots", )
    AC_SUBST(MATLIBS)
    AC_SUBST(MAT_VERSION_FLAG)])


# Find IDL. 9/23/99 jhrg

AC_DEFUN([DODS_IDL], [dnl
    AC_ARG_WITH(idl,
        [  --with-idl=ARG       Where is the IDL root directory],
        IDL_ROOT=${withval}, IDL_ROOT="$IDL_ROOT")
    AC_REQUIRE([AC_CANONICAL_HOST])

    AC_MSG_CHECKING(for the IDL root directory)
    if test -z "$IDL_ROOT"
    then
        # Find IDL's root directory by looking at the exectuable and then 
        # finding where that symbolic link points.
        # !!! Doesn't work if idl isn't a symbolic link - erd !!!
	# I think that the following 'if' fixes the symbolic link problem. 
	# 05/02/03 jhrg 
        idl_loc=`which idl`
	if echo `ls -l $idl_loc` | grep '.*->.*' > /dev/null 2>&1
	then
            idl_loc=`ls -l $idl_loc | sed 's/.*->[ ]*\(.*\)$/\1/'`
	fi
        IDL_ROOT=`echo $idl_loc | sed 's/\(.*\)\/bin.*/\1/'`
    fi

    AC_MSG_RESULT($IDL_ROOT)
    AC_SUBST(IDL_ROOT)

    # Now find where the IDL 5.2 or later sharable libraries live.
    # NB: This won't work if libraries for several architecutures are 
    # installed for several machines.
    AC_MSG_CHECKING(for the IDL sharable library directory)
    # cd to the IDL root because it is likely a symbolic link and find 
    # won't normally follow symbolic links.
    IDL_LIBS=`(cd $IDL_ROOT; find . -name 'libidl.so' -print)`
    # Strip off the leading `.' (it's there because we ran find in the CWD) 
    # and the name of the library used to find the directory.
    IDL_LIBS=`echo $IDL_LIBS | sed 's/\.\(.*\)\/libidl.so/\1/' | sed '1q'`
    IDL_LIBS=${IDL_ROOT}${IDL_LIBS}
    AC_MSG_RESULT($IDL_LIBS)
    AC_SUBST(IDL_LIBS)])


# 5. Misc stuff
#---------------------------------------------------------------------------

# Use the version.h file in the current directory to set the version 
# Makefile varible. All Makefiles should have targets that use this variable
# to rename the directory, build source distribution tarfiles, etc.
AC_DEFUN([DODS_DIRECTORY_VERSION], [dnl
    AC_MSG_CHECKING([version])
    VERSION=`cat version.h`
    AC_MSG_RESULT($VERSION)
    AC_SUBST(VERSION)])

AC_DEFUN([DODS_DEBUG_OPTION], [dnl
    AC_ARG_ENABLE(debug, 
		  [  --enable-debug=ARG      Program instrumentation (1,2)],
		  DEBUG=$enableval, DEBUG=no)

    case "$DEBUG" in
    no) 
      ;;
    1)
      AC_MSG_RESULT(Setting debugging to level 1)
      AC_DEFINE(DODS_DEBUG)
      ;;
    2) 
      AC_MSG_RESULT(Setting debugging to level 2)
      AC_DEFINE(DODS_DEBUG, , [Set instrumentation to level 1 (see debug.h)])
      AC_DEFINE(DODS_DEBUG2, , [Set instrumentation to level 2])
      ;;
    *)
      AC_MSG_ERROR(Bad debug value)
      ;;
    esac])

AC_DEFUN([DODS_CHECK_SIZES], [dnl
    # Ignore the errors about AC_TRY_RUN missing an argument. jhrg 5/2/95

    AC_REQUIRE([AC_PROG_CC])

    AC_CHECK_SIZEOF(int)
    AC_CHECK_SIZEOF(long)
    AC_CHECK_SIZEOF(char)
    AC_CHECK_SIZEOF(double)
    AC_CHECK_SIZEOF(float)

    # Now generate symbols that define the dods_int32, ..., types
    # based on this machine's notion of an int, etc. See dods-datatypes.h.in.
    # I've separated the typedefs from the config_dap.h header because other
    # projects which use the DAP were getting conflicts with their includes,
    # or the includes of still other libraries, and config_dap.h. The 
    # config_dap.h header is now included only by .cc and .c files and headers
    # that need the typedefs use dods-datatypes.h. The code below makes 
    # dods-datatypes.h stand on its own. 8/2/2000 jhrg
    # I changed the order below so that int is tested first. This is more likely
    # to match the type used by the xdr_int() and xdr_u_int() functions. Really,
    # I should test the prototypes but I don't. 3/21/05 jhrg

    if test $ac_cv_sizeof_int -eq 4
    then
	DODS_INT32=int
	DODS_UINT32="unsigned int"
	XDR_INT32=xdr_int
	XDR_UINT32=xdr_u_int
    elif test $ac_cv_sizeof_long -eq 4 
    then
	DODS_INT32=long
	DODS_UINT32="unsigned long"
	XDR_INT32=xdr_long
	XDR_UINT32=xdr_u_long
    else
	AC_MSG_ERROR(How do I get a 32 bit integer on this machine?)
    fi
    # I'm using the three arg form of AC_DEFINE_UNQUOTED because autoheader
    # needs the third argument (although I don't quite get the specifics... 
    # 2/15/2001 jhrg
    AC_DEFINE_UNQUOTED(DINT32, $DODS_INT32, [int32])
    AC_DEFINE_UNQUOTED(DUINT32, $DODS_UINT32, [uint32])
    AC_DEFINE_UNQUOTED(XDR_INT32, $XDR_INT32, [xdr int32])
    AC_DEFINE_UNQUOTED(XDR_UINT32, $XDR_UINT32, [xdr uint32])

    # Assume nobody's hosed the 16-bit types...
    DODS_INT16=short
    DODS_UINT16="unsigned short"
    XDR_INT16=xdr_short
    XDR_UINT16=xdr_u_short
    AC_DEFINE_UNQUOTED(DINT16, $DODS_INT16, [dint16])
    AC_DEFINE_UNQUOTED(DUINT16, $DODS_UINT16, [uint16])
    AC_DEFINE_UNQUOTED(XDR_INT16, $XDR_INT16, [xdr int16])
    AC_DEFINE_UNQUOTED(XDR_UINT16, $XDR_UINT16, [xdr uint16])

    if test $ac_cv_sizeof_char -eq 1
    then
	DODS_BYTE="unsigned char"
    else
	AC_MSG_ERROR(How do I get an 8 bit unsigned integer on this machine?)
    fi
    AC_DEFINE_UNQUOTED(DBYTE, $DODS_BYTE, [dbyte])

    if test $ac_cv_sizeof_double -eq 8
    then
	DODS_FLOAT64=double
	DODS_FLOAT32=float
	XDR_FLOAT64=xdr_double
	XDR_FLOAT32=xdr_float
    else
	AC_MSG_ERROR(How do I get floating point types on this machine?)
    fi
    AC_DEFINE_UNQUOTED(DFLOAT64, $DODS_FLOAT64, [dfloat64])
    AC_DEFINE_UNQUOTED(DFLOAT32, $DODS_FLOAT32, [dfloat32])
    AC_DEFINE_UNQUOTED(XDR_FLOAT64, $XDR_FLOAT64, [xdr float64])
    AC_DEFINE_UNQUOTED(XDR_FLOAT32, $XDR_FLOAT32, [xdr float32])])


